
# $Id: Common.pm,v 1.6 1999/01/06 00:07:21 gtaylor Exp $

package Autobuse::Common;
require Exporter;
@ISA = qw(Exporter);
@EXPORT = qw(get_badports get_badurls
	     format_ip lookup_ip lookup_host paranoid_check_domain);

# URLs we associate with exploits
my @badurls = ("/cgi-bin/phf",
	       "/cgi-bin/test-cgi",
	       "/cgi-bin/handler");
sub get_badurls { return \@badurls; }

# various port numbers and exploit report text
my %badports = ("UDP" => { 19, "connection to the chargen service",
			   111,	"connection to the Sun RPC portmapper",
			   137, "connection to my WINS server",
			   138, "winpopup message",
			   161, "SNMP query",
			   514, "attempt to insert forged log messages",
			   635,	"attempt to contact my NFS mount daemon",
			   640, "attempt to contact my PCNFS service",
			   2049, "attempt to contact my NFS block daemon",
			   },
		"TCP" => { 11, "attempt to use sysstat",
			   15, "attempt to use netstat",
			   19, "attempt to spoof chargen",
			   21, "attempt to ftp into my box",
			   23, "attempt to telnet into my box",
			   25, "attempt to exploit sendmail or relay mail",
			   43, "attempt to connect to whois",
			   53, "attempted bind exploit",
			   79, "attempt to finger my box",
			   110, "attempted POP3 exploit",
			   111, "connection to my portmapper",
			   117, "odd connection to the UUCP port",
			   119, "connection to my nntp service",
			   137, "attempt to use my WINS service",
			   138, "attempt to send a winpopup message",
			   139, "attempt to browse my hard drive via SMB",
			   143, "attempted imapd exploit",
			   144, "attempt to use the NeWS window server",
			   512, "attempt to use the old rexec service",
			   513, "attempt to rlogin",
			   514, "attempt to connect to rshd",
			   515, "attempt to connect to lpd",
			   635, "probe for an exploitable NFS mount daemon",
                           888, "attempt to connect to WinGate admin port",
			   1080, "probe for an open SOCKS server",
			   2049, "probe for an open nfs server",
			   6000, "attempt to connect to X server",
			   12345, "probe for a netbus trojan",
			   },
		"ICMP" => {}	# redirects?  etc

		);

sub get_badports { return \%badports; }

use Socket;

my %ipcache = ();
my %namecache = ();

# Takes a name or a #.#.#.# and returns a name.
sub lookup_ip {
    my $source_ip = $_[0];

    if (defined($source_ip)) {
	if (! $namecache{$source_ip}) {
	    if ($source_ip =~ /^[\.01-9]+$/) {
		$namecache{$source_ip} = gethostbyaddr(inet_aton($source_ip),
						       AF_INET);
	    } else {
		my $addr = gethostbyname($source_ip);
		$namecache{$addr} = $source_ip;
		$source_ip = $addr;
	    }
	}
    }
    return $namecache{$source_ip};
}

# Takes a name or a #.#.#.# and returns a #.#.#.#
sub lookup_host {
    my $source_host = $_[0];

    if (defined($source_host)) {
	if (! $ipcache{$source_host}) {
	    my $addr = gethostbyname($source_host);
	    $ipcache{$source_host} = inet_ntoa($addr);
	}
    }

    return $ipcache{$source_host};
}

sub format_ip {
    my $source_ip = $_[0];

    # Figure out the hostname, if it's in DNS, or just use the no.
    my $source = $source_ip;

    if (defined($source_ip)) {
	my $source_hostname = lookup_ip($source);

	# Return both IP or short name and full DNS name, if we got one
	# back and we didn't just start with the hostname.
	if   (defined($source_hostname) &&
	      $source_hostname ne $source
	      && $source_hostname) {
	    $source = "$source_ip ($source_hostname)";
	}
    }

    return $source;
}

# return true if forward and reverse mappings match any domain name
# in arg 2, a ref to a list of strings
sub paranoid_check_domain {
    my $source_ip = $_[0];
    my $ignore_domains = $_[1];

    # If we have a hostname, be paranoid and then compare
    # against domain names specified in @ignore_domains.
    my $source_hostname = lookup_ip($source_ip);
    if ($source_hostname) {
	for (@{$ignore_domains}) {
	    # either the ignore doamin begins with dot and matches the
	    # end of the hostname, or the ignore domain must match the
	    # whole hostname
	    if (($_ =~ /^[^\.]/  && $_ eq $source_hostname)
		|| 
		($_ =~ /^\./ && $source_hostname =~ /$_$/)) {
		# Check that we get the same IP back.
		# Note that this behaves poorly for names with multiple A recs
		my $ip = gethostbyname($source_hostname);
		if (inet_ntoa($ip) eq $source_ip) {
		    # OK, the source IP and hostname match
		    return 1;
		}
	    }
	}
    }

    return 0;
}

1;
